<?php

/*
 * This file is part of Selenium Tor Driver.
 * (c) R. Eric Wheeler <eric@rewiv.com>
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace Sikofitt\Console\Command;

use GuzzleHttp\Client;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\Process\Process;
use Symfony\Component\Process\ProcessBuilder;

/**
 * Class TorBrowserDownloadCommand
 *
 * @package Sikofitt\Console\Command
 */
class TorBrowserDownloadCommand extends Command
{
    const TOR_VERSION = '6.0.8';
    const TOR_DOWNLOAD_URL = 'https://www.torproject.org/dist/torbrowser';
    const TOR_PACKAGE_LINUX_I386 = 'tor-browser-linux32-6.0.8_en-US.tar.xz';
    const TOR_PACKAGE_LINUX_X64 = 'tor-browser-linux64-6.0.8_en-US.tar.xz';
    const TOR_PACKAGE_WIN = 'torbrowser-install-6.0.8_en-US.exe';

    private $url;

  /**
   * {@inheritdoc}
   */
  public function configure()
  {
      $this->setName('tor:download')
     ->setDescription('Downloads the tor browser.');
  }

  /**
   * Downloads and extracts the tor browser.
   *
   * @param \Symfony\Component\Console\Input\InputInterface   $input
   * @param \Symfony\Component\Console\Output\OutputInterface $output
   * @return void
   */
  public function execute(InputInterface $input, OutputInterface $output)
  {
      $this->url = $this->getDownloadUrl();
      $io = $this->getStyle($input, $output);

      $download = $io->confirm('Download Tor?', true);
      if (false === $download) {
          exit(255);
      }

      $io->note('Downloading ' . $this->getDownloadUrl() . ' ...');
      if (false === file_exists(BASE_DIR . $this->getFileName())) {
          $client = new Client();
          try {
              $data = $client->get($this->url);
          } catch (\Exception $e) {
              $io->error($e->getMessage());
              exit;
          }

          file_put_contents(BASE_DIR . $this->getFileName(), $data->getBody()->getContents());
      }
      $io->success('Downloaded');
      $io->note('Extracting ...');
      $tar = new ProcessBuilder();
      $tar
      ->setPrefix('tar')
      ->setArguments(array('-xvf', BASE_DIR . $this->getFileName()))
      ->getProcess()->run();
  }

  /**
   * Helper method to get symfony style
   *
   * @param \Symfony\Component\Console\Input\InputInterface   $input
   *    The Input instance
   * @param \Symfony\Component\Console\Output\OutputInterface $output
   *    The output instance
   *
   * @return \Symfony\Component\Console\Style\SymfonyStyle
   *    Instance of SymfonyStyle
   */
  private function getStyle(InputInterface $input, OutputInterface $output)
  {
      return new SymfonyStyle($input, $output);
  }

    private function getFileName()
    {
        $arch = $this->getArch();
        $os = $this->getOS();

        switch ($os) {
      case 'linux':
      default:
        switch ($arch) {
          case 'i386':
            $package = self::TOR_PACKAGE_LINUX_I386;
            break;
          case 'x86_64':
          default:
            $package = self::TOR_PACKAGE_LINUX_X64;
            break;
        }
        break;
      case 'win':
      case 'windows':
        $package = self::TOR_PACKAGE_WIN;
        break;
    }
        return $package;
    }

    private function getDownloadUrl()
    {
        $package = $this->getFileName();
        return sprintf('%s/%s/%s', self::TOR_DOWNLOAD_URL, self::TOR_VERSION, $package);
    }

    private function getArch()
    {
        return strtolower(php_uname('m'));
    }

    private function getOS()
    {
        return strtolower(PHP_OS);
    }
}
